#include "spi1.h"
#include <LPC23xx.h>

#include <defines.h>

union Ctrl0_u
{
	struct
	{
		unsigned short DSS : 4;
		unsigned short FRF : 2;
		unsigned short SPO : 1;
		unsigned short SPH : 1;
		unsigned short SCR : 8;
	} bits;
	unsigned short value;
};

void spi1_init(enum spi_mode_e mode, enum spi_wordlen_e wordlen, bool bAutoSelect, uint8_t divider)
{
	union Ctrl0_u ctrl0;
	
	// Disable SSP1
	SSP1CR1 = 0;
	
	switch (mode) {
		case MODE_0:
			ctrl0.bits.SPO = 0;
			ctrl0.bits.SPH = 0;
			break;
		case MODE_1:
			ctrl0.bits.SPO = 0;
			ctrl0.bits.SPH = 1;
			break;
		case MODE_2:
			ctrl0.bits.SPO = 1;
			ctrl0.bits.SPH = 0;
			break;
		case MODE_3:
			ctrl0.bits.SPO = 1;
			ctrl0.bits.SPH = 1;
			break;
	}
	
	ctrl0.bits.FRF = 0;
	ctrl0.bits.SCR = 0;
	ctrl0.bits.DSS = wordlen;
	
	SSP1CR0 = ctrl0.value;
	SSP1CPSR = (divider < 2) ? 2 : divider;
	SSP1CR1 = _BV(1); // enable SPI1
	
	SET_PCB(PINSEL0, 14, PCB_ALT2); // SCK1
	SET_PCB(PINSEL0, 16, PCB_ALT2); // MISO1
	SET_PCB(PINSEL0, 18, PCB_ALT2); // MOSI1
	if (bAutoSelect) {
		SET_PCB(PINSEL0, 12, PCB_ALT2); // SSEL1
	}
	else {
		SETPIN(SPI1_SSEL);
		SET_DDR_OUT(SPI1_SSEL);
	}
}

uint_fast16_t spi1_write(uint_fast16_t data)
{
	SSP1DR = data;
	while (SSP1SR & _BV(4)) { /* empty */ }
	
	return SSP1DR;
}

void spi1_dummyWrite(uint_fast16_t length)
{	
	while (length--) {
		spi1_write(0);
	}
}

void spi1_readBlock(uint8_t* dest, uint_fast16_t length)
{
	while (length--) {
		*dest++ = spi1_write(0);
	}
}

void spi1_writeBlock(uint8_t* src, uint_fast16_t length)
{
	while (length--) {
		spi1_write(*src++);
	}
}

